# -*- coding: utf_8 -*-
#
# SMART TAGS CLASSES

import uno
import unohelper

import re
import sys, os

sys.stdout = open('C:\_log_stdout.txt', 'w')

from com.sun.star.smarttags import XSmartTagRecognizer
from com.sun.star.smarttags import XSmartTagAction
from com.sun.star.lang import XServiceInfo

from com.sun.star.text.TextMarkupType import SMARTTAG as STMARKUP
from com.sun.star.i18n.WordType import ANY_WORD

from com.sun.star.container import NoSuchElementException

#from grammalecte_tags import ACTIONS

EXT_NAME = 'Grammalecte' 

RECOG_IMPL_NAME = "grammalecte.smarttags.TagRecognizer"
RECOG_SERV_NAMES = ("com.sun.star.smarttags.SmartTagRecognizer",)
ACTION_IMPL_NAME = "grammalecte.smarttags.TagAction"
ACTION_SERV_NAMES = (ACTION_IMPL_NAME, "com.sun.star.smarttags.SmartTagAction")

SMARTTAG_NAME = "grammalecte#smarttags"

ACTIONS = [
    [re.compile(ur'\bage\b'), u'âge', u'test1', u'test2'],
    [re.compile(ur'\bmonopôle\b'), 'monopole'],
    [re.compile(ur'\bOOo\b'), 'OO.o', 'OpenOffice.org']
]

class SmartTagsRecog (unohelper.Base, XServiceInfo, XSmartTagRecognizer):    
    def __init__ (self, ctx):
        self.ctx = ctx
        self.svmgr = ctx.ServiceManager
        self.SmartTagCount = 1
        return
    
    # XInitialization
    def initialize (self, objects):
        pass
    
    # XServiceInfo
    def supportsService (self, sName):
        for service in RECOG_SERV_NAMES:
            if service == sName:
                return True
        return False
    
    def getImplementationName (self):
        return RECOG_IMPL_NAME
    
    def getSupportedServiceNames (self):
        return RECOG_SERV_NAMES
    
    # XSmartTagRecognizer
    def getName (self, xLocale):
        return EXT_NAME
    
    def getDescription (self, xLocale):
        return 'Reconnaissance des SmartTags'  if xLocale.Language == 'fr'  else 'SmartTags Recognition'
    
    def getSmartTagName (self, nSmartTagIndex):
        # SmartTag name is of the format "namespaceURI#tagname"
        return SMARTTAG_NAME
    
    def getSmartTagDownloadURL (self, nSmartTagIndex):
        return ''
    
    def recognize (self, sText, nStart, nLength, eDataType, xLocale, xTextMarkup, sAppName, xController, xTokenizer):
        if xLocale.Language != 'fr' and sAppName != 'Calc':
            return
        nEndPos = nStart + nLength
        #aWord = [""]
        #aBound = xTokenizer.getWordBoundary(sText, nStart, xLocale, ANY_WORD, True)
        #while aBound.startPos < aBound.endPos and aBound.endPos <= nEndPos:
        #    aWord[0] = sText[aBound.startPos:aBound.endPos]
        #    match = self.PathExp.match(sText[aBound.startPos:aBound.endPos])
        #    if match:
        #        xTextMarkup.commitTextMarkup(STMARKUP, SMARTTAG_NAME, aBound.startPos, aBound.endPos-aBound.startPos, None)
        #    aBound = xTokenizer.nextWord(sText, aBound.startPos, xLocale, ANY_WORD)
        for action in ACTIONS:
            for match in action[0].finditer(sText[nStart:nEndPos]):
                xStringKeyMap = xTextMarkup.getMarkupInfoContainer()
                for i in range(1, len(action)):
                    xStringKeyMap.insertValue('ctxMenuST_'+str(i-1), action[i])
                xTextMarkup.commitTextMarkup(STMARKUP, SMARTTAG_NAME, nStart+match.start(), match.end()-match.start(), xStringKeyMap)
        return
    
    def hasPropertyPage (self, nSmartTagIndex, xLocale):
        return False 
    
    def displayPropertyPage (self, nSmartTagIndex, xLocale):
        return
    

class SmartTagsDo (unohelper.Base, XServiceInfo, XSmartTagAction):    
    def __init__ (self, ctx):
        self.ctx = ctx
        self.svmgr = ctx.ServiceManager
        self.SmartTagCount = 1
        self.ActionCount = 1
        return
    
    # XInitialization
    def initialize (self, objects):
        pass
    
    # XServiceInfo
    def supportsService (self, sName):
        for service in ACTION_SERV_NAMES:
            if service == sName:
                return True
        return False
    
    def getImplementationName (self):
        return ACTION_IMPL_NAME
    
    def getSupportedServiceNames (self):
        return ACTION_SERV_NAMES
    
    # XSmartTagAction
    def getName (self, xLocale):
        return EXT_NAME
    
    def getDescription (self, xLocale):
        return 'Action sur les SmartTags'  if xLocale.Language == 'fr'  else 'SmartTags Actions'
    
    def getSmartTagName (self, nSmartTagIndex):
        # SmartTag name is of the format "namespaceURI#tagname"
        return SMARTTAG_NAME
    
    def getSmartTagCaption (self, nSmartTagIndex, xLocale):
        return 'Confusion possible '
    
    def getActionCount (self, sSmartTagName, xController):
        return self.ActionCount
    
    def getActionID (self, sSmartTagName, nActionIndex, xController):
        nActionID = nActionIndex
        return nActionID
    
    def getActionNameFromID (self, nActionID, xController):
        return str(nActionID)
        
    def getActionCaptionFromID (self, nActionID, sAppName, xLocale, xProperties, sText, sXML, xController, xTarget):
        self.ActionCount = xProperties.Count
        sKey = 'ctxMenuST_' + str(nActionID)
        sVar = xProperties.getValue(sKey)  if xProperties.hasValue(sKey)  else '## error ##'
        return 'Remplacez par : ' + sVar
    
    def invokeAction (self, nActionID, sAppName, xController, xTarget, xProperties, sText, sXML, xLocale):
        sKey = 'ctxMenuST_' + str(nActionID)
        sNewString = xProperties.getValue(sKey)  if xProperties.hasValue(sKey)  else '## error ##'
        xTarget.setString(sNewString)
        return
    
    def isCaptionDynamic (self, nActionID, sAppName, xController, xLocale):
        return True
    
    def isShowSmartTagIndicator (self, nActionID, sAppName, xController, xLocale):
        return True


g_ImplementationHelper = unohelper.ImplementationHelper()
g_ImplementationHelper.addImplementation(SmartTagsRecog, RECOG_IMPL_NAME, RECOG_SERV_NAMES)
g_ImplementationHelper.addImplementation(SmartTagsDo, ACTION_IMPL_NAME, ACTION_SERV_NAMES)
